//****************************************************************
// Modulname:    $Source: $
// User:         $Author: $ MEBA
// Version:      $Name: $  $Revision: $ 0.9
// Datum:        $Date: $ 01.01.2001
// Qualitaet:    $State: $ 
//****************************************************************
// Beschreibung: Librar-Routinen fr den IC-Bus
//****************************************************************
// Historie:
//==========
// $Log: $
//****************************************************************
// Steuerparameter:
//****************************************************************
#pragma
//****************************************************************
// globale Definitionen
//****************************************************************
#define I2C_LIB_C

//****************************************************************
// verwendete Include Dateien
//****************************************************************
#include <defines.h>

// enthaelt allgemeine Einstellungen zum Projekt
#include <defines.h>
#include <globals.h> 
#include <intrins.h>
#include <I2C_Chip_Lib.h>
#include <I2C_DEF.h>        
#include <I2C_LIB.h>        

//****************************************************************
// Deklarationen von Funktionen und Variablen
//****************************************************************
/* Variablendefinitionen */
bdata uchar bdValue;
 sbit D0_I2C = bdValue^0;
 sbit D1_I2C = bdValue^1;
 sbit D2_I2C = bdValue^2;
 sbit D3_I2C = bdValue^3;
 sbit D4_I2C = bdValue^4;
 sbit D5_I2C = bdValue^5;
 sbit D6_I2C = bdValue^6;
 sbit D7_I2C = bdValue^7;

//****************************************************************
// Definitionen von Variablen
//****************************************************************

//****************************************************************
// Funktionsname uc_StartI2C() 
// Version: 1.0    Datum 
// Uebergabeparameter: keine
// Beschreibung: 
// Die Funktion uc_StartI2C() fuehrt eine Startbedingung fuer den
// I2C-Bus durch. Dabei wird falls noetig zuerst eine Stopbedingung 
// ein geleitet.   
// Rueckgabewerte:
//   OK       es ist kein Fehler aufgetreten                               
//   ERR_SCL_SET/_CLEAR  SCL laesst sich nicht auf HIGH/LOW setzen. 
//   ERR_SDA_SET/_CLEAR  SDA laesst sich nicht auf HIGH/LOW setzen. 
//****************************************************************
uchar uc_StartI2C(void){
  uchar ucError = OK;
  uchar ucLoop =0;

  do {
    SCL_SET; // Startbedingung schaffen
    btSDA = SET;
    ucLoop++;
  } while ((!btSCL || !btSDA) && ucLoop < MAX_ZYK);

  if (!btSCL) ucError = ERR_SCL_SET; // Signal geht nicht auf HIGH
  if (!btSDA) ucError = ERR_SDA_SET; //   "           "        "

 if(!ucError) // Stopbedingung vorhanden
   SDA_CLEAR;
    if (btSDA) ucError = ERR_SDA_CLEAR; // Signal geht nicht auf LOW

   SCL_CLEAR;
    if (btSCL) ucError = ERR_SCL_CLEAR; // Signal geht nicht auf LOW

 if (ucError) ucError = ERR_START_I2C | ucError;
 return(ucError);
}

//****************************************************************
// Funktionsname uc_StopI2C() 
// Version: 1.0    Datum 
// Uebergabeparameter: keine
// Beschreibung: 
// Die Funktion uc_StopI2C() erzeugt eine Stoppbedingung auf dem
// I2C-Bus durch.
// Rueckgabewerte:
//   OK       es ist kein Fehler aufgetreten                               
//   ERR_SCL_SET/_CLEAR  SCL laesst sich nicht auf HIGH/LOW setzen. 
//   ERR_SDA_SET/_CLEAR  SDA laesst sich nicht auf HIGH/LOW setzen. 
//****************************************************************
uchar uc_StopI2C(void){
  uchar ucError = OK;
  uchar ucLoop =0;

  
  SDA_CLEAR;
  do {
    SCL_SET; // Endebedingung
    ucLoop++;
  } while (!btSCL && ucLoop < MAX_ZYK);

  if (!btSCL) ucError = ERR_SCL_CLEAR;
  else { 
   btSDA = SET;
   if (btSDA == CLEAR) { // Das Lesen mit ACK abgeschlossen
     SCL_CLEAR;
     for (ucLoop=0; ucLoop < 10; ucLoop++) {
      SCL_SET; SCL_CLEAR; // DATEN austackten
     }
     SDA_CLEAR;   // Endebedingung einleiten
     SCL_SET;
   }  
   ucLoop =0;
    do {
      btSDA = SET; // Endebedingung
      ucLoop++;
    } while (!btSDA && ucLoop < MAX_ZYK);
    if (!btSDA) ucError = ERR_SDA_CLEAR;
  }
  if (ucError) ucError = ERR_STOP_I2C | ucError;
  return(ucError);
}

//****************************************************************
// Funktionsname uc_WriteI2C() 
// Version: 1.0    Datum 
// Uebergabeparameter: keine
// Beschreibung: 
// Die Funktion uc_WriteI2C() schreibt einen Wert auf den I2C-Bus.
// Nach dem 8ten Bit wird das Acknowlege-Signal ueberprueft. Wird
// ACK nicht erkannt, wird eine Fehlermeldung zurueckgegeben
// Rueckgabewerte:
//   OK       es ist kein Fehler aufgetreten                               
//   ERR_ACK  es ist kein ACK erkannt worden
//****************************************************************
uchar uc_WriteI2C(uchar ucMode, uchar ucValue){
  uchar ucError = OK;

  bdValue = ucValue;

// Die Definitionen fuer die Macros SCL_CLEAR und  SCL_SET finden 
// Sie im H-File I2C_DEF.H. Die Wait-Zeiten werden in Abhaengigkeit 
// von SYSTEM_CLK definiert.

  btSDA = D7_I2C; SCL_SET; SCL_CLEAR;   // 1 Taktzyklus (MSB)
  btSDA = D6_I2C; SCL_SET; SCL_CLEAR;   // 2 Taktzyklus
  btSDA = D5_I2C; SCL_SET; SCL_CLEAR;   // 3 Taktzyklus
  btSDA = D4_I2C; SCL_SET; SCL_CLEAR;   // 4 Taktzyklus
  btSDA = D3_I2C; SCL_SET; SCL_CLEAR;   // 5 Taktzyklus
  btSDA = D2_I2C; SCL_SET; SCL_CLEAR;   // 6 Taktzyklus
  btSDA = D1_I2C; SCL_SET; SCL_CLEAR;   // 7 Taktzyklus
  btSDA = D0_I2C; SCL_SET; SCL_CLEAR;   // 8 Taktzyklus (LSB)

  btSDA = SET; // Datenfreigabe fuer ACK-Signal
  SCL_SET; // ACK-Takt
   if (btSDA) ucError = ERR_ACK;
  SCL_CLEAR;
   if (ucMode == RE_START) btSCL = SET;
   else btSDA = CLEAR;

 if (ucError) ucError = ERR_WRITE_I2C | ucError;
  return(ucError);
}

//****************************************************************
// Funktionsname uc_ReadI2C() 
// Version: 1.0    Datum 
// Uebergabeparameter: keine
// Beschreibung: 
// Die Funktion uc_WriteI2C() liet einen Wert auf den I2C-Bus.
// Mit dem Bit btAck wird die Ausgabe von Acknowlege gesteuert.
// Rueckgabewerte:
//   empfangene Wert
//****************************************************************
uchar uc_ReadI2C(bit btACK){

  btSDA = SET; // Freigabe vom BUS

  SCL_SET; D7_I2C = btSDA; SCL_CLEAR; // MSB
  SCL_SET; D6_I2C = btSDA; SCL_CLEAR;
  SCL_SET; D5_I2C = btSDA; SCL_CLEAR;
  SCL_SET; D4_I2C = btSDA; SCL_CLEAR;
  SCL_SET; D3_I2C = btSDA; SCL_CLEAR;
  SCL_SET; D2_I2C = btSDA; SCL_CLEAR;
  SCL_SET; D1_I2C = btSDA; SCL_CLEAR;
  SCL_SET; D0_I2C = btSDA; SCL_CLEAR;

 if (btACK == ACK) btSDA = CLEAR; // ACK vom Master
  btSCL = SET;
  SCL_SET; SCL_CLEAR; // ACK-Takt 

  return(bdValue);
}

//****************************************************************
// Funktionsname uc_I2C_Dev() 
// Version: 1.0    Datum 
// Uebergabeparameter: Bausteinadresse
// Beschreibung: 
// Die Funktion uc_I2C_Dev() ueberprueft, ob ein Baustein sich auf
// uebergebenen Adresse befindet. Die Ueberpruefung erfolgt durch 
// das Acknowlege-Signal.
// Rueckgabewerte:
//   OK = Baustein auf der Adresse gefunden
//****************************************************************
uchar uc_I2C_Dev(uchar ucI2C_Dev){
  uchar ucError = OK;

// 1. Startbedingung setzen
  ucError = uc_StartI2C();
// 2. Anwahl des I2C-Bus Bausteins  (R/W = Schreiben)
  if (ucError == OK) ucError = uc_WriteI2C(I2C_DEV,ucI2C_Dev);
// 3. Erneuter Versuch Anwahl des I2C-Bus Bausteins 
  if (ucError != OK) ucError = uc_WriteI2C(I2C_DEV,ucI2C_Dev);
// 4. Stoppbedingung setzen
  if (ucError == OK) ucError = uc_StopI2C();
  else uc_StopI2C(); // stoppen ohne Fehlermeldung zu ueberschreiben
  return(ucError);
}

//****************************************************************
// Funktionsname uc_Data2I2C() 
// Version: 1.0    Datum 
// Uebergabeparameter: Bausteinadresse, Adresslage im Baustein,
// Startadresse vom LV-Objekt
// Beschreibung: 
// Die Funktion uc_Data2I2C() schreibt Daten in einen Baustein.
// Rueckgabewerte:
//   OK = Baustein korrekt beschrieben
//   ACK-/ SCL- /SDA-Fehler
//****************************************************************
uchar uc_Data2I2C(uchar ucI2CDev, uchar ucI2CAddr, uchar *pucSrc) {
  uchar ucError = OK;
  uchar ucAnzahl = *pucSrc;
  pucSrc++;

// 1. Startbedingung setzen
  ucError = uc_StartI2C();
// 2. Anwahl des I2C-Bus Bausteins  (R/W = Schreiben)
  if (ucError == OK) ucError = uc_WriteI2C(I2C_DEV,ucI2CDev);
// 3. Setzen der Adresse von der ab geschrieben werden soll
  if (ucError == OK) ucError = uc_WriteI2C(I2C_ADR,ucI2CAddr); 

// 4. Anzahl Bytes schreiben
  if (ucError == OK)
   for (; ucAnzahl; ucAnzahl--) {
      ucError = uc_WriteI2C(I2C_DATA,*pucSrc); // Daten schreiben
      if (ucError != OK) break;
      pucSrc++;
   }
// 5. Stoppbedingung setzen
  if (ucError == OK) ucError = uc_StopI2C();
    else uc_StopI2C(); // stoppen ohne Fehlermeldung zu ueberschreiben

// 6. Falls Der Baustein ein EEPROM ist, wird hier das Busy abgefragt
 do { 
   ucError =  uc_I2C_Dev(ucI2CDev);
 } while (ucError != OK);

 return(ucError);
}

//****************************************************************
// Funktionsname uc_I2C2Data() 
// Version: 1.0    Datum 
// Uebergabeparameter: Bausteinadresse, Adresslage im Baustein,
// Startadresse vom LV-Objekt
// Beschreibung: 
// Die Funktion uc_I2C2Data() liest Daten aus einem Baustein.
// Rueckgabewerte:
//   OK = Baustein korrekt ausgelesen
//   ACK-/ SCL- /SDA-Fehler
//****************************************************************
uchar uc_I2C2Data(uchar ucI2CDev, uchar ucI2CAddr, uchar *pucDes) {
  uchar ucError;
  uchar ucAnzahl;

  ucAnzahl = *pucDes; // Liest die Anzahl der zu lesenden Bytes aus
 
// 1. Startbedingung setzen
  ucError = uc_StartI2C();	  
// 2. Anwahl des I2C-Bus Bausteins  (R/W = Schreiben)
  if (ucError == OK) ucError = uc_WriteI2C(I2C_DEV,ucI2CDev);   
// 3. Setzen der Adresse von der ab ausgelesen werden soll
  if (ucError == OK) ucError = uc_WriteI2C(RE_START,ucI2CAddr);
// 4. erneute Startbedingung setzen 
  if (ucError == OK) ucError = uc_StartI2C();				    
// 5. Anwahl des I2C-Bus Bausteins  (R/W = Lesen)
  if (ucError == OK) ucError = uc_WriteI2C(I2C_DEV,ucI2CDev | READ_I2C);

// 6. Anzahl Bytes auslesen
  for (; ucAnzahl; ucAnzahl--) {
    if (ucAnzahl == 1) 
      *pucDes = uc_ReadI2C(NACK); // Daten lesen ohne Acknowlege zu senden
    else
      *pucDes = uc_ReadI2C(ACK);  // Daten lesen mit Acknowlege
    pucDes++;
  }
// 7. Stoppbedingung setzen
  if (ucError == OK) ucError = uc_StopI2C();
  else uc_StopI2C(); // stoppen ohne Fehlermeldung zu ueberschreiben
  return(ucError);
}

//****************************************************************
// Funktionsname uc_Data2I2C_EEPROM() 
// Version: 1.0    Datum 
// Uebergabeparameter: Bausteinadresse, Adresslage im Baustein,
// Startadresse vom LV-Objekt
// Beschreibung: 
// Die Funktion uc_Data2I2C_EEPROM() schreibt Daten in einen EEPROM
// Dabei wird die Pageadressierung bercksichtigt.
// Rueckgabewerte:
//   OK = Baustein korrekt beschrieben
//   ACK-/ SCL- /SDA-Fehler
//****************************************************************
#if (EEPROM_FUNKTION == SET)
uchar uc_Data2I2C_EEPROM(uchar ucI2CDev, uchar ucI2CAddr, uchar *pucSrc) {
  uchar ucError = OK;
  uchar ucAnzahl, ucAnzahl1;
  ucAnzahl = ucAnzahl1 = *pucSrc;
  pucSrc++;


// 4. Sind Bytes zum Schreiben vorhanden ??
   if (ucAnzahl){ 
     if (ucAnzahl < 8){
WRITE_NORMAL:
     // 4.1 Startbedingung setzen
       ucError = uc_StartI2C();
     // 4.2 Anwahl des I2C-Bus Bausteins  (R/W = Schreiben)
       if (ucError == OK) ucError = uc_WriteI2C(I2C_DEV,ucI2CDev);
     // 4.3 Setzen der Adresse von der ab geschrieben werden soll
       if (ucError == OK) ucError = uc_WriteI2C(I2C_ADR,ucI2CAddr); 

     // 4.4 Es werden bis zu 7 Bytes geschrieben
       if (ucError == OK) 
         for (; ucAnzahl; ucAnzahl--) {
            ucError = uc_WriteI2C(I2C_DATA,*pucSrc); // Daten schreiben
            if (ucError != OK) break;
            pucSrc++;
         }
     // 4.5 Stoppbedingung setzen
       if (ucError == OK) ucError = uc_StopI2C();
       else uc_StopI2C(); // stoppen ohne Fehlermeldung zu ueberschreiben

     // 4.6 Falls Der Baustein ein EEPROM ist, wird hier das Busy abgefragt
       do { 
         ucError =  uc_I2C_Dev(ucI2CDev);
       } while (ucError != OK);
     }
// 5. Pagebetrieb
     if(ucAnzahl1 >= 8){
       if (ucAnzahl){
	     ucAnzahl = ucI2CAddr & B00000111;
         // 5.1 Schreiben der ersten Bytes bis zu Pagegrenze
         if (ucAnzahl) {
           goto WRITE_NORMAL;
         }
         ucI2CAddr += ucAnzahl;
         ucAnzahl1 -= ucAnzahl;
         ucAnzahl  = ucAnzahl1 % EEP_PAGE_SIZE; 
         ucAnzahl1 = ucAnzahl1 / EEP_PAGE_SIZE; 
         // Es wird der Page-Betrieb eingesetzt
         if (ucAnzahl1){ 
           for (; ucAnzahl1; ucAnzahl1--) {
             uchar ucLoop;
           // 5.2 Startbedingung setzen
             ucError = uc_StartI2C();
           // 5.3 Anwahl des I2C-Bus Bausteins  (R/W = Schreiben)
             if (ucError == OK) ucError = uc_WriteI2C(I2C_DEV,ucI2CDev);
           // 5.4 Setzen der Adresse von der ab geschrieben werden soll
             if (ucError == OK) ucError = uc_WriteI2C(I2C_ADR,ucI2CAddr); 
           // 5.5 Es werden bis zu 7 Bytes geschrieben
             if (ucError == OK) 
               for (ucLoop = EEP_PAGE_SIZE; ucLoop; ucLoop--){
                 ucError = uc_WriteI2C(I2C_DATA,*pucSrc); // Daten schreiben
                 if (ucError != OK) break;
                pucSrc++;
               }
               ucI2CAddr += EEP_PAGE_SIZE;      
           // 5.6 Stoppbedingung setzen
             if (ucError == OK) ucError = uc_StopI2C();
             else uc_StopI2C(); // stoppen ohne Fehlermeldung zu ueberschreiben
           // 5.7 Falls Der Baustein ein EEPROM ist, wird hier das Busy abgefragt
             do { 
               ucError =  uc_I2C_Dev(ucI2CDev);
             } while (ucError != OK);
           }
           if (ucAnzahl) {  // Die restlichen Bytes (<8) schreiben.
             goto WRITE_NORMAL;
           }
		 }
       } 
       
     }
   }

 return(ucError);
}
#endif
//****************************************************************
// Funktionsname uc_IO2I2C() 
// Version: 1.0    Datum 
// Uebergabeparameter: Bausteinadresse, 
// Beschreibung: 
// Die Funktion uc_IO2I2C() liest Datum aus einem I/O-Expander.
// Die Funktion ist fuer den PCF8574(A) angepasst. Bei anderen Bau-
// steinen mu eventuell noch eine Anpassung vorgenommen werden.
// Rueckgabewerte:
//   OK = Baustein korrekt ausgelesen
//   ACK-/ SCL- /SDA-Fehler
//****************************************************************
uchar uc_IO2I2C(uchar ucI2CDev, uchar *pucDes){
  uchar ucError = OK;

// 1. Startbedingung setzen
  ucError = uc_StartI2C();
// 2. Anwahl des I2C-Bus Bausteins  (R/W = Lesen)
  if (ucError == OK) ucError = uc_WriteI2C(I2C_DEV,ucI2CDev | READ_I2C);
// 3. Datum auslesen
  if (ucError == OK) *pucDes = uc_ReadI2C(NACK); 
// 4. Stoppbedingung setzen
  if (ucError == OK) ucError = uc_StopI2C();
  else uc_StopI2C(); // stoppen ohne Fehlermeldung zu ueberschreiben
  return(ucError);
}

//****************************************************************
// Funktionsname uc_IO2I2C() 
// Version: 1.0    Datum 
// Uebergabeparameter: Bausteinadresse, 
// Beschreibung: 
// Die Funktion uc_IO2I2C() schreibt ein Datum aus einem I/O-Expander.
// Die Funktion ist fuer den PCF8574(A) angepasst. Bei anderen Bau-
// steinen mu eventuell noch eine Anpassung vorgenommen werden.
// Rueckgabewerte:
//   OK = Baustein korrekt ausgelesen
//   ACK-/ SCL- /SDA-Fehler
//****************************************************************
uchar uc_I2C2IO(uchar ucI2CDev, uchar ucValue){
  uchar ucError = OK;

// 1. Startbedingung setzen
  ucError = uc_StartI2C();
// 2. Anwahl des I2C-Bus Bausteins  (R/W = Schreiben)
  if (ucError == OK) ucError = uc_WriteI2C(I2C_DEV,ucI2CDev);
// 3. Datum schreiben
  if (ucError == OK) ucError = uc_WriteI2C(I2C_DATA,ucValue);
// 4. Stoppbedingung setzen
  if (ucError == OK) ucError = uc_StopI2C();
  else uc_StopI2C(); // stoppen ohne Fehlermeldung zu ueberschreiben
  return(ucError);
}
