/*
   Titel  : Echtzeituhr Header fr PCF8583
   Autor  : Gnther Pregartner, 5AIH 2000/2001
   Version: 1.1
*/

#include "i2c.h"

#ifndef RTC
#define RTC

unsigned char _hex2dec( unsigned char hex )
{
   int hex2=(int)hex;
   int ret=0;

   ret +=((int)(hex2/100)*256);
   hex2-=((int)(hex2/100)*100);
   ret +=((int)(hex2/10)*16);
   hex2-=((int)(hex2/10)*10);
   ret +=((int)(hex2));

   return ret;
}

unsigned char _dec2hex( unsigned char dec )
{
   int dec2=(int)dec;
   int ret=0;

   ret +=((int)(dec2/256)*100);
   dec2-=((int)(dec2/256)*256);
   ret +=((int)(dec2/16)*10);
   dec2-=((int)(dec2/16)*16);
   ret +=((int)(dec2));

   return ret;
}

unsigned char rtc_exists()
{
   return i2c_exists( 0xA0 );
}

void rtc_setTime( unsigned char hour, unsigned char min, unsigned char sec, unsigned char hund )
{
   i2c_start();
   i2c_write( 0xA0 );      // Slave Adress
   i2c_write( 0x01 );      // set pointer to control status
   i2c_write( _hex2dec(hund) );
                           // to hundreth of a second

   i2c_start();
   i2c_write( 0xA0 );      // Slave Adress
   i2c_write( 0x02 );      // set pointer to control status
   i2c_write( _hex2dec(sec) );
                           // to hundreth of a second

   i2c_start();
   i2c_write( 0xA0 );      // Slave Adress
   i2c_write( 0x03 );      // set pointer to control status
   i2c_write( _hex2dec(min) );
                           // to hundreth of a second

   i2c_start();
   i2c_write( 0xA0 );      // Slave Adress
   i2c_write( 0x04 );      // set pointer to control status
   i2c_write( _hex2dec(hour) );
                           // to hundreth of a second

   i2c_stop();
}

void rtc_getTime( unsigned char * hour, unsigned char * min,
                  unsigned char * sec, unsigned char * hund )
{

   i2c_start();
   i2c_write( 0xA0 );      // Address
   i2c_write( 0x01 );      // set data pointer
   i2c_start();
   i2c_write(0xA1);        // enable read mode
   *hund = _dec2hex( i2c_read(1) );

   i2c_start();
   i2c_write( 0xA0 );      // Address
   i2c_write( 0x02 );      // set data pointer
   i2c_start();
   i2c_write(0xA1);        // enable read mode
   *sec = _dec2hex( i2c_read(1) );

   i2c_start();
   i2c_write( 0xA0 );      // Address
   i2c_write( 0x03 );      // set data pointer
   i2c_start();
   i2c_write(0xA1);        // enable read mode
   *min  = _dec2hex( i2c_read(1) );

   i2c_start();
   i2c_write( 0xA0 );      // Address
   i2c_write( 0x04 );      // set data pointer
   i2c_start();
   i2c_write(0xA1);        // enable read mode
   *hour = _dec2hex( i2c_read(1) & 0x3F );

   i2c_stop();
}

void rtc_writeRAM( unsigned char dptr, unsigned char dat )
{
   i2c_start();
   i2c_write( 0xA0 );
   i2c_write( dptr );
   i2c_write( dat );
   i2c_stop();
}

void rtc_readRAM( unsigned char dptr, unsigned char * dat )
{
   i2c_start();
   i2c_write( 0xA0 );
   i2c_write( dptr );
   i2c_start();
   i2c_write( 0xA1 );
   *dat = i2c_read( 1 );
   i2c_stop();
}

void rtc_getDate( unsigned char * day, unsigned char * month, unsigned char * weekday )
{
   unsigned char d1;

   i2c_start();
   i2c_write( 0xA0 );      // Address
   i2c_write( 0x05 );      // set data pointer
   i2c_start();
   i2c_write(0xA1);        // enable read mode
   d1 = i2c_read(1);
   *day = (((d1 & 0x30)>>4)*10) + (d1 & 0x0F);

   i2c_start();
   i2c_write( 0xA0 );      // Address
   i2c_write( 0x06 );      // set data pointer
   i2c_start();
   i2c_write(0xA1);        // enable read mode
   d1 = i2c_read(1);
   *month = (((d1 & 0x10)>>4)*10) + (d1 & 0x0F);
   *weekday = (d1 & 0xE0)>>5;

   i2c_stop();            
}

void rtc_setDate( unsigned char day, unsigned char month, unsigned char weekday, unsigned char schaltjahr )
{
   unsigned char d1;

   i2c_start();
   i2c_write( 0xA0 );      // Address
   i2c_write( 0x05 );      // set data pointer
   i2c_write( (((int)(day/10))<<4) +
              (schaltjahr<<6) +
                     (day%10) );

   i2c_start();
   i2c_write( 0xA0 );      // Address
   i2c_write( 0x06 );      // set data pointer
   i2c_write( (((int)(month/10))<<4) +
                        (month%10) +
                        (weekday<<5) );

   i2c_stop();            
      
}

#endif
