/************************************************************************/
/*                            bung 17                                  */
/*                    Interruptgesteuerte Uhr                           */
/*                                                                      */
/************************************************************************/

/****   Steueranweisungen an den Compiler       ****/
#pragma iv(0x8000)
#pragma mod517
#pragma code debug pl(61)

/****   Angabe der Include Dateien              ****/
#include <reg517.h>
#include <stdio.h>
#include "lcd.h"

#define uchar unsigned char
#define uint unsigned int

/****   Globale Variblen                        ****/
bit secflag;
uchar stu, min, sec, msec;
char buffer[21];

/****   Timer Interrupt 0 wird von der          ****/
/****   von der Zeitschleife ausgelst          ****/
/****   und erzeugt alle 100 ms einen Interrupt ****/

void INTTIM0 (void) interrupt 1
{
        TH0 = -39;
        TL0 = 0;
        
        msec++;
        if ( msec == 100 )
                {
                msec = 0;
                sec++;
                secflag = 1;
                }
       if ( sec == 60 )
                {
                sec = 0;
                min++;
                }
       if ( min == 60 )
                {
                min = 0;
                stu++;
                }
       if ( stu == 24 ) stu = 0;
}


/****   Prototypen der Funktionen               ****/
void set_time (uchar stunde, uchar minute, uchar secunde, uchar msecunde);
void ini_ser0(void);
void ini_ser1(void);

/****   Funktionen                              ****/
void set_time (uchar stunde, uchar minute, uchar secunde, uchar msecunde)
{
        stu = stunde;
        min = minute;
        sec = secunde;
        msec = msecunde;
}

/************************************************************************/
/*      function ini_ser0 initialisiert serielle Schnittstelle 0        */
/************************************************************************/

void ini_ser0(void)
{
        data unsigned int OLD;

        S0CON = 0x72; /* Serieller Mode 1, 8bit, 1 Stopbit, TI0 gesetzt */
        BD = 1;                 /* Baudrate enable                      */
        OLD = PCON;             /* Baudrate 9600 bit                    */
        PCON = OLD | 0x80;      /* SMOD = 1                             */
}

/************************************************************************/
/*      function ini_ser1 initialisiert serielle Schnittstelle 1        */
/************************************************************************/

void ini_ser1(void)
{
        S1CON = 0xF2;   /* Serieller Mode B, 8bit, 1 Stopbit, TI0 gesetzt */
        S1REL = 0xD9;   /* Ladewert 217 fuer 9600 baud                  */
}

/************************************************************************/
/*                      Hauptprogramm                                   */
/************************************************************************/

main ()
{

TMOD = 0x01;            /**** Timer 0 Mode 1 16 Bit             ****/
TH0  = -39;             /**** Vorgesetzter Wert, alle 10 ms Int.****/
TL0  = 0;               /**** Vorgesetzter Wert                 ****/
ET0 = 1;                /**** Timer 0 Interrupt disabled        ****/
EAL = 1;                /**** Interrupt allgemein erlaubt       ****/

TR0 = 1;                /**** Einschalten der Zeitschleife      ****/

init_lcd();
blank_lcd();
ini_ser0();
ini_ser1();

set_time (20,30,0,0);

while (1)
        {
        
        while (!secflag);
        secflag = 0;
        printf ("%2u:%2u:%2u \n",(uint)stu,(uint)min,(uint)sec);
        sprintf(buffer,"Uhr mit Timer 0 Int.");
        print_lcd(1,1,buffer);
        sprintf (buffer,"%2u:%2u:%2u",(uint)stu,(uint)min,(uint)sec); 
        print_lcd(2,4,buffer);
        
        }

}

