/****************************************************************************
  I2C.C

  Funktion: IC Bus Funktionen

*****************************************************************************/

/*** Compiler Commands ***/
#pragma debug pagelength (60)
#pragma noiv

/*** Include Files ***/
#include <reg517.h>     /* Definitions for 80C552 register */
#include <stdio.h>      /* Standard In/Out-Library */

/*** Define Statements ***/
#define WT 2            /* Wait-Time (min 4,3 sec) - see IC timeing protocol */

// extern bit lcd_ser;

sbit SCL = P3^5;
sbit SDA = P3^4;

/************************************************
*  Wait XXX msec                                *
*  wait()                                       *
*************************************************/
void wait (int msec)
{
  char i;

	for (msec; msec!=0; msec--)
		for (i=0; i<83; i++)	 /* 12 mikrosec. each cycle */
			;   		 /* 12 * 83 = ca. 1 msek */

}



/************************************************
*  Initialisize IC bus to transmit data        *
*  i2c_start()                                  *
*************************************************/
void i2c_start()
{
  /* SCL = 1;wait(1); */

  SDA = 0;   /* Serial Data Line IC Bus */
   wait(WT); /* Wait min. 4,7 sec - see IC timing protocol PCF8583 */
  SCL = 0;   /* Serial Clock Line IC Bus */
   wait(WT);

}


/************************************************
*  Stop data transmission at IC bus            *
*  i2c_stop()                                   *
*************************************************/
void i2c_stop()
{

  SDA = 0;   /* Serial Data Line IC Bus */
   wait(WT);
  SCL = 1;   /* Serial Clock Line IC Bus */
   wait(WT);
  SDA = 1;
   wait(WT);

}


/************************************************
*  Transmit byte to slave receiver              *
*  i2c_trans()                                  *
*************************************************/
void i2c_trans(unsigned char zeich)
{
  unsigned char i;
  unsigned char mask = 0x80; /* Bit mask */

/*printf("\n ->Transmit: ");*/


  /*** Transmission on IC Bus starts with the MSB
       and stops with the LSB of the byte to transmit ***/
  for (i=0;i<8;i++)
     {
	SDA = ((zeich & mask) == mask) ? 1:0;  /* Decode MSB at Serial Data Line IC Bus ... */

/*  printf("%1d|",(int)SDA);    */

	 mask >>= 1;         /* ... and shift right */
	 wait(WT);
	SCL = 1;             /* Serial Clock Line IC Bus */
	 wait(WT);
	SCL = 0;
     }

  /*** Send Clock Acknowledge after one byte was transmitted ***/

     SDA = 1;    /* Data Line HIGH during one clock-pulse for acknowledge */
     SCL = 1;
      wait(WT);
     SCL = 0;
      wait(WT);

}



/************************************************
*  Receive byte from slave transmitter          *
*  i2c_rec()                                    *
*************************************************/

unsigned char i2c_rec()
{ unsigned char zeich;
  unsigned char i;
  unsigned char mask = 0x80;   /* Bit mask */
   zeich = 0;     /* Received byte */


  /*** Receiving on IC Bus starts with the MSB
       and stops with the LSB of the received byte ***/
  for (i=0;i<8;i++)
     {
	SCL = 1;   /* Serial Clock Line IC Bus */
	 wait(WT);

	zeich = (int)SDA ? (mask | zeich) : zeich;   /* Decode received byte ... */
	mask >>= 1;                                  /* ... and shift right */

	SCL = 0;
	 wait(WT);
     }

  return(zeich);

}



/************************************************
*  Send acknowledge after received byte         *
*    ( request for next byte )                  *
*  i2c_ack()                                    *
*************************************************/
void i2c_ack()
{

  /*** Data Line LOW during one clock-pulse for acknowledge ***/
     SDA = 0;   /* Serial Data Line IC Bus */
      wait(WT);
     SCL = 1;   /* Serial Clock Line IC Bus */
      wait(WT);
     SCL = 0;
      wait(WT);
     SDA = 1;
      wait(WT);

}



/************************************************
*  Send 'no' acknowledge after received byte    *
*  i2c_no_ack()                                 *
*************************************************/
void i2c_no_ack()
{

  /*** Data Line HIGH during one clock-pulse for 'no' acknowledge ***/
     SDA = 1;   /* Serial Data  Line IC Bus */
      wait(WT);
     SCL = 1;   /* Serial Clock Line IC Bus */
      wait(WT);
     SCL = 0;
      wait(WT);

}

/* EOF: I2C.C */

